/* TICKOS.H:  OS/2 alarm clock server - constants, prototypes & structures

    03-28-93 EAS
*/

enum { APPNOTRESPONDING, APPBUSY, APPREJECTEDDATA };

#define szClientClass        "WC_CLIENT"
#define szObjClass           "WC_OBJ"

#define szTopic              "CLOCK"
#define szItemName           "TIME"

#define idFrameWindow        100

#define IDM_FILE             300
#define IDM_EXIT             301
#define IDM_HELP             302
#define IDM_ABOUT            303
#define IDM_ALLERRORS        304          // report all error menu option
#define IDLB_LISTBOX         320

#define UM_LINKERROR         WM_USER+1    // link error user msg
#define UM_CLIENTLINKED      WM_USER+2
#define UM_LINKMESSAGE       WM_USER+3    // link status user msg
#define UM_ALLERRORS         WM_USER+4    // report all errors user msg

#define FALLERRORS           0x00001000   // report all error flag

typedef struct _DATETIME FAR *LPDT;
enum { HOUR, MINUTE, SECOND, MONTH, DAY, YEAR };

PCHAR pszAppName;                 // Application name

/////////////////////////////////////////// OBJIDA //////////////////
// Server's object window Instance Data Area (IDA)
struct OBJIDA {
  OBJIDA(HWND client, HWND sender, PCHAR appname,
         PCHAR topic)
         : pszItemName(0), bAdvise(FALSE), bNotifyUpdate(FALSE),
           bAcknowledgeRequest(FALSE), bAcknowledgementPending(FALSE),
           usTimerNo(TID_USERMAX-1), uflags(0), pflags(0),
           bAllErrors(FALSE) {
    hwndClient = client;
    hwndSender = sender;
    pszAppName = new char[strlen(appname)+1];
    strcpy(pszAppName, appname);
    pszTopic = new char[strlen(topic)+1];
    strcpy(pszTopic, topic);
  }
  ~OBJIDA(VOID) {
    delete pszAppName;
    delete pszTopic;
  }

  HWND    hwnd;                   // object window's handle
  HWND    hwndClient;             // server's client window handle
  HWND    hwndSender;             // sender's object window handle
  PCHAR   pszAppName;             // application name
  PCHAR   pszTopic;               // topic (database) name
  PCHAR   pszItemName;            // item (field) name
  USHORT  usTimerNo;              // timer number
  BOOL    bAdvise;                // FALSE = request, TRUE = advise
  BOOL    bAcknowledgeRequest;    // server will set DDE_FACKREQ bit in
                                  // the transmitted DDESTRUCT to indicate
                                  // that it will wait for a WM_DDE_ACK
                                  // response from the sender before it
                                  // ships the next data update.
  BOOL    bAcknowledgementPending;// TRUE if we have not received an
                                  // WM_DDE_ACK from sender from last
                                  // WM_DDE_ADVISE data shipment. Only
                                  // tested if 'acknowledgeRequest' is TRUE.
  BOOL    bNotifyUpdate;          // If the is TRUE, during WM_DDE_ADVISE
                                  // processing, the server sends 0-length
                                  // data DDESTRUCT packet to signal
                                  // updated data instead of actually
                                  // sending the data. The sender is
                                  // expected to get the data via a
                                  // WM_DDE_REQUEST message.
  USHORT  uflags;
  USHORT  pflags;
  BOOL    bAllErrors;             // Report all errors = TRUE

  // Application specific data
  INT     interval;               // data update interval
  CHAR    szText[80];             // messaging buffer
  USHORT  usAlarm;                // alarm if
  USHORT  usDuration;             // alarm duration
  HWND    hwndListbox;            // linked client listbox handle
};
typedef OBJIDA *POBJIDA;


//////////////////////////////////////////// CLIENTIDA //////////////
// Server's client window Instance Data Area (IDA)
struct CLIENTIDA {
  CLIENTIDA(HWND hwndClient) : hwndObj(0) { hwnd=hwndClient; }
    CHAR          szText[80];   // text that appears status line
    HWND          hwnd;         // client area handle
    HWND          hwndObj;      // object window handle
    HWND          hwndListbox;  // listbox handle
    PCHAR         pszAppName;   // instance server name = EXE file name
    SHORT         foreground;   // client area foreground color
    SHORT         background;   // client background color
    FONTMETRICS   fontMetrics;
    static USHORT usInstance;   // links counter
    USHORT        usFlags;      // utility flags:
                                //   0xooo1 oooo  = report all errors
};
typedef CLIENTIDA  *PCLIENTIDA;


//////////////////////////////////////////// CMDSTRUCT //////////////
// WM_DDE_EXECUTE data command structure
struct CMDSTRUCT {
  CHAR   szCmdStr[6];      // Alarm ID
  CHAR   separator[1];     // for the comma separator
  CHAR   szDuration[7];    // Alarm duration
};
typedef CMDSTRUCT *PCMDSTRUCT;


// Prototypes
MRESULT EXPENTRY clientWndProc(HWND, ULONG, MPARAM, MPARAM);
MRESULT EXPENTRY objWndProc(HWND, ULONG, MPARAM, MPARAM);
VOID makeLink(HWND, HWND);
PDDESTRUCT EXPENTRY createDataPacket(HWND hwnd,
                                     PSZ pszItemName,
                                     USHORT fsStatus  = 0,
                                     PSZ pszData      = NULL,
                                     USHORT usDataLen = 0,
                                     USHORT usFormat  = DDEFMT_TEXT);
PCHAR EXPENTRY getTime(VOID);
PCHAR getDate(VOID);
VOID alarmThread(PVOID);
BOOL setListboxText(POBJIDA);
BOOL toggleCheck(HWND, ULONG);
VOID enableMenuItem(HWND, USHORT, BOOL);

