/* TICKOSC.H: DDE Alarm Clock testbed client
*/

enum { TIME, INTERVAL, ALARM };

// strings
#define szClientClass      "WC_CLIENT"
#define szObjClass         "WC_OBJ"
#define szAppName          "TICKOS"         // Application/server name
#define szTopic            "CLOCK"          // topic/record name
#define DEFINTERVAL        "1000"           // default update interval

// colors
#define clrForeground      CLR_BLACK
#define clrBackground      CLR_CYAN

#define TID_INITIATE        TID_USERMAX-1   // init timeout value
#define TID_REQUEST         TID_USERMAX-2   // request timeout value

// menu item IDs
#define rid                 100             // resource id
#define IDM_DDE             300
#define IDM_INIT            301
#define IDM_ONE             302
#define IDM_POKE            303
#define IDM_START           304
#define IDM_STOP            305
#define IDM_TERM            306
#define IDM_EXIT            307
#define IDM_HELP            308
#define IDM_ABOUT           309

#define IDM_CLOCK           310
#define IDM_ALARMS          311
#define IDM_DATE            312
#define IDM_TIME            313

// user defined messages
#define UM_REQUEST           WM_USER+1  // get a data update
#define UM_START             WM_USER+2  // start continuous update
#define UM_STOP              WM_USER+3  // stop continuous update
#define UM_TERM              WM_USER+4  // terminate link
#define UM_POKE              WM_USER+5  // sets update interval
#define UM_NOFREE            WM_USER+6  // toggle don't delete packet
#define UM_RETRY             WM_USER+7  // toggle retry message send
#define UM_EXECUTE           WM_USER+8  //
#define UM_DDE_TIMEOUT       WM_USER+9  // request-for-data timeout
#define UM_LINKERROR         WM_USER+10

#define IDM_SETUP            400  // Setup menu option
#define IDM_NOFREE           410  // server deletes data packets
#define IDM_SETSERVER        420  // set/change server name
#define IDM_SETTOPIC         430  // set/change topic/record name
#define IDM_SETITEM          435  // set/change item/field name
#define IDM_EDITALARMS       440  // suggestive 1
#define IDM_ALARM1           441  // execute alarm 1
#define IDM_ALARM2           442  // execute alarm 2
#define IDM_ALARM3           443  // execute alarm 3
#define IDM_ALARM4           444  // execute alarm 4
#define IDM_HIDECLOCK        450  // suggestive 2
#define IDM_HIDECONTROLS     460  // suggestive 3
#define IDM_RETRY            470  // retry if server queue full
#define IDM_FACKREQ          480  // Client acknowledges data receipt
#define  UM_FACKREQ          481
#define IDM_FNODATA          490  // server signals data availability
#define  UM_FNODATA          491  // by shipping zero-length data packet
#define  UM_LINKINITFAILURE  492  // attempt to establish DDE link failed
#define  UM_NOSERVERRESPONSE 493  // server not responding
#define  UM_LINKED           494  // link established

#define  DLG_INTERVAL	       497  // interval dialog box
#define  DLGF_INTERVAL       498  // interval dialog box - frame
#define  DLGEF_INTERVAL      499  // interval entry field

#define  DLG_ALARMS	         500  // Alarms dialog box
#define  DLGF_ALARMS	       501  // alarms dialog box - frame
#define  DLGL_ALARMS	       502  // alarms list box

#define  SERVERNOTRESPONDING 600  // server not responding

// Application defined status bits; high order byte of fsStatus
// DDESTRUCT word.
#define  DDE_FAPP_FACKREQ    0x0100
#define  DDE_FAPP_NOFREE     0x0200  // I'm using this bit to let the
                                     // server know whether the appli-
                                     // cation is deleteing the data
                                     // packets or not. It is a mystery
                                     // to me why DDEPM_NOFREE is defined
                                     // for the 'ulOptions' argument of
                                     // WinDdePostMsg() but I can't find
                                     // any mechanism for detecting this
                                     // bit in any receiver machinery???

//////////////////////////////////////////// OBJIDA /////////////////
struct OBJIDA {
  OBJIDA(VOID) { hwndClient = 0; hwndServer = 0; }
  HWND   hwndClient;      // Client window handle
  HWND   hwndServer;      // Server window handle
  CHAR   szappname[9];    // Application/server name
  CHAR   sztopic[30];     // topic/record name
  USHORT usInitiate;      // If non-zero, flags a DDE initialization
                          // in effect and is the time permitted to
                          // establish the link (in milliseconds) before
                          // a "Can't establish link" message is
                          // displayed.
  USHORT usRequest;       // If non-zero, flags a DDE single shot data
                          // update operation in effect and its value
                          // is the time permitted to aqcuire data
                          // before a "Server is not responding" message
                          // is displayed. If 0, flags a continuous
                          // data update operation in effect
  USHORT uflags;          // Utility flags: the bits in this dodad
                          // control whether:
                          //   - client retries message transmission at
                          //     1-second intervals until message is
                          //     successfully received.
                          //   - server or client deletes DDE data packet
                          //     (see DDE_FAPP_NOFREE #defined above)
  USHORT pflags;          // Protocol flags: the bits in this dodad
                          // control whether:
                          //   - Application will acknowledge data receipt
                          //   - Server will send WM_DDE_DATA packets
                          //     with 0-length data to signal data
                          //     available
};
typedef OBJIDA *POBJIDA;

//////////////////////////////////////////// CLIENTIDA //////////////
struct CLIENTIDA {
  CLIENTIDA(VOID) { hwndObj = 0L; }
  HWND hwndMenu;              // client's top level menu handle
  HWND hwndObj;               // object window handle
  CHAR szTime[20];            // time string
  CHAR szStatus[80];          // status composition buffer
  CHAR szLink[20];            // link status (No link/init/linked)
  CHAR szMsgType[12];         // Message type (Advise/Request) string
  CHAR szRETRY[20];           // Retry status string (RETRY)
  CHAR szNOFREE[20];          // Server deletes status string (NOFREE)
  CHAR szFACKREQ[20];         // (FACKREQ), (FNODATA)
  CHAR szFNODATA[20];
  CHAR szInterval[10];        // update interval string
  USHORT usDDEflags;          // protocol & utility flags
};
typedef CLIENTIDA *PCLIENTIDA;

// prototypes
MRESULT EXPENTRY clientWndProc(HWND, ULONG, MPARAM, MPARAM);
MRESULT EXPENTRY objWndProc(HWND, ULONG, MPARAM, MPARAM);
VOID setDDE_Controls(HWND, BOOL state=FALSE);
VOID initObjWindow(HWND);
VOID setTitle(HWND);
PDDESTRUCT EXPENTRY createDataPacket(HWND hwnd,
                                     PSZ pszItemName,
                                     USHORT fsStatus  = 0,
                                     PSZ pszData      = NULL,
                                     USHORT usDataLen = 0,
                                     USHORT usFormat  = DDEFMT_TEXT);
BOOL toggleCheck(HWND, ULONG);
VOID enableMenuItem(HWND, USHORT, BOOL);
MRESULT EXPENTRY intervalDlgProc(HWND, ULONG, MPARAM, MPARAM);

